///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.array;



import java.util.Random;



/******************************************************************************
Fills arrays.

<p><b>Details:</b> <b>ArrayFiller</b> is a collection of functions for filling arrays with values.  In each of the fill methods, if the array passed in is null, no action is taken and no exception is thrown.</p>

@author Sharky
******************************************************************************/

public class ArrayFiller
{



	/**********************************************************************
	Fills an array with the specified <code>Object</code>.  If <code>array</code> is null, no action is taken and no exception is thrown.  If <code>array</code>'s type prohibits storing <code>obj</code> in its elements, an <code>ArrayStoreException</code> is thrown.

	@param array the array
	@param obj the object
	@exception ArrayStoreException if <code>array</code> cannot hold <code>obj</code>
	**********************************************************************/

	public static void fill (Object [] array, Object obj)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = obj;
	}



	/**********************************************************************
	Fills an array of <code>byte</code>s with the specified value.

	@param array the array
	@param value the value
	**********************************************************************/

	public static void fill (byte [] array, byte b)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = b;
	}



	/**********************************************************************
	Fills an array of <code>short</code>s with the specified value.

	@param array the array
	@param value the value
	**********************************************************************/

	public static void fill (short [] array, short value)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = value;
	}



	/**********************************************************************
	Fills an array of <code>int</code>s with the specified value.

	@param array the array
	@param value the value
	**********************************************************************/

	public static void fill (int [] array, int value)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = value;
	}



	/**********************************************************************
	Fills an array of <code>long</code>s with the specified value.

	@param array the array
	@param value the value
	**********************************************************************/

	public static void fill (long [] array, long value)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = value;
	}



	/**********************************************************************
	Fills an array of <code>float</code>s with the specified value.

	@param array the array
	@param value the value
	**********************************************************************/

	public static void fill (float [] array, float value)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = value;
	}



	/**********************************************************************
	Fills an array of <code>double</code>s with the specified value.

	@param array the array
	@param value the value
	**********************************************************************/

	public static void fill (double [] array, double value)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = value;
	}



	/**********************************************************************
	Fills an array of <code>char</code>s with the specified value.

	@param array the array
	@param value the value
	**********************************************************************/

	public static void fill (char [] array, char value)
	{
		if (array == null)
			return;
		for (int i = 0; i < array . length; ++ i)
			array [i] = value;
	}



}



