///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.array;



/******************************************************************************
256 byte array.

<p><b>Details:</b>  This class is a dynamically allocated 256-byte page in the virtual array mechanism described in {@link ByteArray32 ByteArray32}.</p>

@see ByteArray32
******************************************************************************/

final class ByteArray08
{



	private byte default_byte;



	private byte[] data;



	ByteArray08 (byte default_byte)
	{
		this . default_byte = default_byte;
	}



	byte get (int index)
	{
		if (data == null)
			return default_byte;
		return data [index & 0xff];
	}



	void set (int index, byte value)
	{
		if (data == null)
		{
			if (value == default_byte)
				return;
			data = new byte [0x100];
			for (int i = 0; i < 0x100; ++ i)
				data [i] = default_byte;
		}
		data [index & 0xff] = value;
	}



	void get
	(
		int index,
		byte[] dest,
		int dest_from,
		int amount
	)
	{
		if (data == null)
		{
			for (int i = 0; i < amount; ++ i)
				dest [dest_from + i] = default_byte;
			return;
		}
		index &= 0xff;
		for (int i = 0; i < amount; ++ i)
			dest [dest_from + i] = data [index + i];
	}



	void set
	(
		int index,
		byte[] src,
		int src_from,
		int amount
	)
	{
		index &= 0xff;
		if (data == null)
		{
			data = new byte [0x100];
			for (int i = 0; i < index; ++ i)
				data [i] = default_byte;
			for (int i = index + amount; i < 0x100; ++ i)
				data [i] = default_byte;
		}
		for (int i = 0; i < amount; ++ i)
			data [index + i] = src [src_from + i];
	}



	int getIntBe (int index)
	{
		if (data == null)
		{
			int i = default_byte & 0xff;
			return i << 24 | i << 16 | i << 8 | i;
		}
		index &= 0xfc;
		return
			(data [index    ] & 0xff) << 24
		|	(data [index + 1] & 0xff) << 16
		|	(data [index + 2] & 0xff) <<  8
		|	(data [index + 3] & 0xff)      ;
	}



	int getIntLe (int index)
	{
		if (data == null)
		{
			int i = default_byte & 0xff;
			return i << 24 | i << 16 | i << 8 | i;
		}
		index &= 0xfc;
		return
			(data [index    ] & 0xff)
		|	(data [index + 1] & 0xff) <<  8
		|	(data [index + 2] & 0xff) << 16
		|	(data [index + 3] & 0xff) << 24;
	}



	void setIntBe (int index, int value)
	{
		int b0 = value >>> 24;
		int b1 = value >> 16 & 0xff;
		int b2 = value >> 8 & 0xff;
		int b3 = value & 0xff;
		if (data == null)
		{
			if (b0 == b1 && b1 == b2 && b2 == b3 && b3 == (default_byte & 0xff))
				return;
			data = new byte [0x100];
			for (int i = 0; i < 0x100; ++ i)
				data [i] = default_byte;
		}
		index &= 0xfc;
		data [index    ] = (byte) b0;
		data [index + 1] = (byte) b1;
		data [index + 2] = (byte) b2;
		data [index + 3] = (byte) b3;
	}



	void setIntLe (int index, int value)
	{
		int b3 = value >>> 24;
		int b2 = value >> 16 & 0xff;
		int b1 = value >> 8 & 0xff;
		int b0 = value & 0xff;
		if (data == null)
		{
			if (b0 == b1 && b1 == b2 && b2 == b3 && b3 == (default_byte & 0xff))
				return;
			data = new byte [0x100];
			for (int i = 0; i < 0x100; ++ i)
				data [i] = default_byte;
		}
		index &= 0xfc;
		data [index    ] = (byte) b0;
		data [index + 1] = (byte) b1;
		data [index + 2] = (byte) b2;
		data [index + 3] = (byte) b3;
	}



	void fill (int index, int amount, byte value)
	{
		index &= 0xff;
		if (index == 0 && amount == 0x100)
		{
			data = null;
			default_byte = value;
			return;
		}
		if (data == null)
		{
			if (value == default_byte)
				return;
			data = new byte [0x100];
			for (int i = 0; i < index; ++ i)
				data [i] = default_byte;
			for (int i = index + amount; i < 0x100; ++ i)
				data [i] = default_byte;
		}
		for (int i = 0; i < amount; ++ i)
			data [index + i] = value;
	}



	int search (int index, int amount, byte value)
	{
		index &= 0xff;
		if (data == null)
		{
			if (value == default_byte && amount > 0)
				return index;
			return -1;
		}
		for (int i = 0; i < amount; ++ i)
			if (data [index + i] == value)
				return index + i;
		return -1;
	}



	// returns default byte if compaction succeeded, -1 otherwise
	int compact ()
	{
		if (data == null)
			return default_byte;
		final int b = data [0];
		for (int i = 1; i < 0x100; ++ i)
			if (b != data [i])
				return -1;
		return b & 0xff;
	}



}



