///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.array;



final class ByteArray24
{



	private static final int PAGE_ADDR_SIZE = 16;



	private static final int PAGE_SIZE = 1 << PAGE_ADDR_SIZE;



	private static final int ROUND_MASK = -1 << PAGE_ADDR_SIZE;



	private static final int ADDRESS_MASK = ~ (ROUND_MASK << 8);



	private static int get_page_number (int index)
	{
		return index >>> PAGE_ADDR_SIZE & 0xff;
	}



	private byte default_byte;



	private int default_int;



	private void set_default_int ()
	{
		int b = default_byte & 0xff;
		default_int = b << 24 | b << 16 | b << 8 | b;
	}



	private ByteArray16[] data;



	public ByteArray24 (byte default_byte)
	{
		this . default_byte = default_byte;
		set_default_int ();
	}



	public byte get (int index)
	{
		if (data == null)
			return default_byte;
		int page = get_page_number (index);
		if (data [page] == null)
			return default_byte;
		else
			return data [page] . get (index);
	}



	public void set (int index, byte value)
	{
		if (data == null)
			if (value == default_byte)
				return;
			else
				data = new ByteArray16 [0x100];
		int page = get_page_number (index);
		if (data [page] == null)
			if (value == default_byte)
				return;
			else
				data [page] = new ByteArray16 (default_byte);
		data [page] . set (index, value);
	}



	public void get
	(
		int index,
		byte[] dest,
		int dest_from,
		int amount
	)
	{
		if (data == null)
		{
			for (int i = 0; i < amount; ++ i)
				dest [dest_from + i] = default_byte;
			return;
		}
		int page = get_page_number (index);
		while (true)
		{
			final int next_index = (index & ROUND_MASK) + PAGE_SIZE;
			final int new_amount = Math.min (amount, next_index - index);
			if (data [page] == null)
				for (int i = 0; i < new_amount; ++ i)
					dest [dest_from + i] = default_byte;
			else
				data [page] . get (index, dest, dest_from, new_amount);
			amount -= new_amount;
			if (amount == 0)
				return;
			index = next_index;
			dest_from += new_amount;
			++ page;
		}
	}



	public void set
	(
		int index,
		byte[] src,
		int src_from,
		int amount
	)
	{
		if (data == null)
			data = new ByteArray16 [0x100];
		int page = get_page_number (index);
		while (true)
		{
			final int next_index = (index & ROUND_MASK) + PAGE_SIZE;
			final int new_amount = Math.min (amount, next_index - index);
			if (data [page] == null)
				data [page] = new ByteArray16 (default_byte);
			data [page] . set (index, src, src_from, new_amount);
			amount -= new_amount;
			if (amount == 0)
				break;
			index = next_index;
			src_from += new_amount;
			++ page;
		}
	}



	public int getIntBe (int index)
	{
		int page = get_page_number (index);
		if (data == null || data [page] == null)
			return default_int;
		return data [page] . getIntBe (index);
	}



	public int getIntLe (int index)
	{
		int page = get_page_number (index);
		if (data == null || data [page] == null)
			return default_int;
		return data [page] . getIntLe (index);
	}



	public void setIntBe (int index, int value)
	{
		if (data == null)
			if (value == default_int)
				return;
			else
				data = new ByteArray16 [0x100];
		int page = get_page_number (index);
		if (data [page] == null)
			if (value == default_int)
				return;
			else
				data [page] = new ByteArray16 (default_byte);
		data [page] . setIntBe (index, value);
	}



	public void setIntLe (int index, int value)
	{
		if (data == null)
			if (value == default_int)
				return;
			else
				data = new ByteArray16 [0x100];
		int page = get_page_number (index);
		if (data [page] == null)
			if (value == default_int)
				return;
			else
				data [page] = new ByteArray16 (default_byte);
		data [page] . setIntLe (index, value);
	}



	public void fill (int index, int amount, byte value)
	{
		if (data == null)
			if (value == default_byte)
				return;
			else
				data = new ByteArray16 [0x100];
		int page = get_page_number (index);
		while (true)
		{
			final int next_index = (index & ROUND_MASK) + PAGE_SIZE;
			final int new_amount = Math.min (amount, next_index - index);
			if (new_amount == PAGE_SIZE)
				if (value == default_byte)
					data [page] = null;
				else
					data [page] = new ByteArray16 (value);
			else
			{
				if (data [page] == null)
					data [page] = new ByteArray16 (default_byte);
				data [page] . fill (index, new_amount, value);
			}
			amount -= new_amount;
			if (amount == 0)
				break;
			index = next_index;
			++ page;
		}
	}



	public int search (int index, int amount, byte value)
	{
		if (amount <= 0)
			return -1;
		if (data == null)
			if (value == default_byte)
				return index & ADDRESS_MASK;
			else
				return -1;
		int page = get_page_number (index);
		while (true)
		{
			final int next_index = (index & ROUND_MASK) + PAGE_SIZE;
			final int new_amount = Math.min (amount, next_index - index);
			if (data [page] == null)
			{
				if (value == default_byte)
					return index & ADDRESS_MASK;
			}
			else
			{
				int result = data [page] . search (index, new_amount, value);
				if (result != -1)
					return page << PAGE_ADDR_SIZE | result;
			}
			amount -= new_amount;
			if (amount == 0)
				break;
			index = next_index;
			++ page;
		}
		return -1;
	}



	public int compact ()
	{
		if (data == null)
			return default_byte;
		int b;
		if (data [0] == null)
			b = default_byte;
		else
			b = data [0] . compact ();
		for (int i = 1; i < 0x100; ++ i)
		{
			int c;
			if (data [i] == null)
				c = default_byte;
			else
			{
				c = data [i] . compact ();
				if (c == (default_byte & 0xff))
					data [i] = null;
			}
			if (b != c)
				b = -1;
		}
		if (b >= 0)
		{
			default_byte = (byte) b;
			set_default_int ();
			data = null;
		}
		return b;
	}



}



