///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



import lava.security.IBlockCipher64;



/******************************************************************************
Encrypts using DES-ECB.

<p><b>Details:</b> <code>DesEcbEncoder</code> encrypts <code>long</code> arrays using DES (or 3DES) in ECB mode.  <strong>Please refer to the superclass documentation for important information regarding usage.</strong></p>

@since 2001.02.02
@author Sharky
******************************************************************************/

public class DesEcbEncoder extends DesEcbCoder
{



	/**********************************************************************
	Sets encryption engine.

	<p><b>Details:</b>  This constructor sets the 64-bit block encryption engine (<var>engine</var>) used for encryption.  <var>engine</var> can be an instance of <code>DesEngine</code>, <code>TripleDesEngine</code>, or any other 64-bit block cipher that satisfies the <code>IBlockCipher64</code> interface contract.</p>

	@param engine the engine
	**********************************************************************/

	public DesEcbEncoder (IBlockCipher64 engine)
	{
		super (engine);
	}



	/**********************************************************************
	Encrypts longs.

	<p><b>Details:</b>  <code>continueTransform</code> encrypts the plain text stored in <var>src</var> and stores the resulting cipher text in the provided output array (<var>dest</var>).  Both arrays must be <code>long</code> arrays.  If <var>dest</var> is too short to store the result, the overflow is discarded.</p>

	@param src plain text
	@param dest cipher text
	@param length number of plain text symbols
	@return number of cipher text symbols stored or discarded

	@version 2000.03.06
	**********************************************************************/

	public int continueTransform (Object src, Object dest, int length)
	{
		long[] vplSrc = (long[]) src;
		long[] vplDst = (long[]) dest;
		try
		{
			engine . encrypt (vplSrc, vplDst, length);
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			if (length <= vplSrc . length && vplDst . length < length)
				return length;
			throw e;
		}
		return length;
	}



}



